import attrs


@attrs.define(frozen=True)
class Gitmoji:
    """Class that represents a gitmoji."""

    type: str
    icon: str
    code: str
    desc: str

    @property
    def value(self) -> str:
        """The value property."""
        return f"{self.icon} {self.type}"

    @property
    def name(self) -> str:
        """The name property."""
        return f"{self.icon} {self.type}: {self.desc}"



from shared.gitmojis import *

mojis = [
    {
        "type": "fix",
        "icon": GJ_FIX,
        "code": ":bug:",
        "desc": "Fix a bug.",
    },
    {
        "type": "feat",
        "icon": GJ_FEAT,
        "code": ":sparkles:",
        "desc": "Introduce new features.",
    },
    {
        "type": "docs",
        "icon": GJ_DOCS,
        "code": ":memo:",
        "desc": "Add or update documentation.",
    },
    {
        "type": "style",
        "icon": GJ_STYLE,
        "code": ":art:",
        "desc": "Improve structure / format of the code.",
    },
    {
        "type": "refactor",
        "icon": GJ_REFACTOR,
        "code": ":recycle:",
        "desc": "Refactor code.",
    },
    {
        "type": "perf",
        "icon": GJ_PERF,
        "code": ":zap:",
        "desc": "Improve performance.",
    },
    {
        "type": "test",
        "icon": GJ_TEST,
        "code": ":white_check_mark:",
        "desc": "Add, update, or pass tests.",
    },
    {
        "type": "build",
        "icon": GJ_BUILD,
        "code": ":construction_worker:",
        "desc": "Add or update CI build system.",
    },
    {
        "type": "ci",
        "icon": GJ_CI,
        "code": ":green_heart:",
        "desc": "Fix CI Build.",
    },
    {
        "type": "revert",
        "icon": GJ_REVERT,
        "code": ":rewind:",
        "desc": "Revert changes.",
    },
    {
        "type": "dump",
        "icon": GJ_DUMP,
        "code": ":fire:",
        "desc": "Remove code or files.",
    },
    {
        "type": "hotfix",
        "icon": GJ_HOTFIX,
        "code": ":ambulance:",
        "desc": "Critical hotfix.",
    },
    {
        "type": "deploy",
        "icon": GJ_DEPLOY,
        "code": ":rocket:",
        "desc": "Deploy stuff.",
    },
    {
        "type": "ui",
        "icon": GJ_UI,
        "code": ":lipstick:",
        "desc": "Add or update the UI and style files.",
    },
    {
        "type": "init",
        "icon": GJ_INIT,
        "code": ":tada:",
        "desc": "Begin a project.",
    },
    {
        "type": "security",
        "icon": GJ_SECURITY,
        "code": ":lock:",
        "desc": "Fix security issues.",
    },
    {
        "type": "secret",
        "icon": GJ_SECRET,
        "code": ":closed_lock_with_key:",
        "desc": "Add or update secrets.",
    },
    {
        "type": "bump",
        "icon": GJ_BUMP,
        "code": ":bookmark:",
        "desc": "Release / Version tags.",
    },
    {
        "type": "fix-lint",
        "icon": GJ_FIX_LINT,
        "code": ":rotating_light:",
        "desc": "Fix compiler / linter warnings.",
    },
    {
        "type": "wip",
        "icon": GJ_WIP,
        "code": ":construction:",
        "desc": "Work in progress.",
    },
    {
        "type": "dep-drop",
        "icon": GJ_DEP_DROP,
        "code": ":arrow_down:",
        "desc": "Downgrade dependencies.",
    },
    {
        "type": "dep-bump",
        "icon": GJ_DEP_BUMP,
        "code": ":arrow_up:",
        "desc": "Upgrade dependencies.",
    },
    {
        "type": "pin",
        "icon": GJ_PIN,
        "code": ":pushpin:",
        "desc": "Pin dependencies to specific versions.",
    },
    {
        "type": "analytics",
        "icon": GJ_ANALYTICS,
        "code": ":chart_with_upwards_trend:",
        "desc": "Add or update analytics or track code.",
    },
    {
        "type": "dep-add",
        "icon": GJ_DEP_ADD,
        "code": ":heavy_plus_sign:",
        "desc": "Add a dependency.",
    },
    {
        "type": "dep-rm",
        "icon": GJ_DEP_RM,
        "code": ":heavy_minus_sign:",
        "desc": "Remove a dependency.",
    },
    {
        "type": "config",
        "icon": GJ_CONFIG,
        "code": ":wrench:",
        "desc": "Add or update configuration files.",
    },
    {
        "type": "script",
        "icon": GJ_SCRIPT,
        "code": ":hammer:",
        "desc": "Add or update development scripts.",
    },
    {
        "type": "lang",
        "icon": GJ_LANG,
        "code": ":globe_with_meridians:",
        "desc": "Internationalization and localization.",
    },
    {
        "type": "typo",
        "icon": GJ_TYPO,
        "code": ":pencil2:",
        "desc": "Fix typos.",
    },
    {
        "type": "poop",
        "icon": GJ_POOP,
        "code": ":poop:",
        "desc": "Write bad code that needs to be improved.",
    },
    {
        "type": "merge",
        "icon": GJ_MERGE,
        "code": ":twisted_rightwards_arrows:",
        "desc": "Merge branches.",
    },
    {
        "type": "package",
        "icon": GJ_PACKAGE,
        "code": ":package:",
        "desc": "Add or update compiled files or packages.",
    },
    {
        "type": "external",
        "icon": GJ_EXTERNAL,
        "code": ":alien:",
        "desc": "Update code due to external API changes.",
    },
    {
        "type": "resource",
        "icon": GJ_RESOURCE,
        "code": ":truck:",
        "desc": "Move or rename resources (e.g.: files, paths, routes).",
    },
    {
        "type": "license",
        "icon": GJ_LICENSE,
        "code": ":page_facing_up:",
        "desc": "Add or update license.",
    },
    {
        "type": "boom",
        "icon": GJ_BOOM,
        "code": ":boom:",
        "desc": "Introduce breaking changes.",
    },
    {
        "type": "asset",
        "icon": GJ_ASSET,
        "code": ":bento:",
        "desc": "Add or update assets.",
    },
    {
        "type": "accessibility",
        "icon": GJ_ACCESSIBILITY,
        "code": ":wheelchair:",
        "desc": "Improve accessibility.",
    },
    {
        "type": "source-docs",
        "icon": GJ_SOURCE_DOCS,
        "code": ":bulb:",
        "desc": "Add or update comments in source code.",
    },
    {
        "type": "beer",
        "icon": GJ_BEER,
        "code": ":beers:",
        "desc": "Write code drunkenly.",
    },
    {
        "type": "text",
        "icon": GJ_TEXT,
        "code": ":speech_balloon:",
        "desc": "Add or update text and literals.",
    },
    {
        "type": "db",
        "icon": GJ_DB,
        "code": ":card_file_box:",
        "desc": "Perform database related changes.",
    },
    {
        "type": "logs-add",
        "icon": GJ_LOGS_ADD,
        "code": ":loud_sound:",
        "desc": "Add or update logs.",
    },
    {
        "type": "logs-rm",
        "icon": GJ_LOGS_RM,
        "code": ":mute:",
        "desc": "Remove logs.",
    },
    {
        "type": "people",
        "icon": GJ_PEOPLE,
        "code": ":busts_in_silhouette:",
        "desc": "Add or update contributor(s).",
    },
    {
        "type": "ux",
        "icon": GJ_UX,
        "code": ":children_crossing:",
        "desc": "Improve user experience / usability.",
    },
    {
        "type": "arch",
        "icon": GJ_ARCH,
        "code": ":building_construction:",
        "desc": "Make architectural changes.",
    },
    {
        "type": "design",
        "icon": GJ_DESIGN,
        "code": ":iphone:",
        "desc": "Work on responsive design.",
    },
    {
        "type": "mock",
        "icon": GJ_MOCK,
        "code": ":clown_face:",
        "desc": "Mock things.",
    },
    {
        "type": "egg",
        "icon": GJ_EGG,
        "code": ":egg:",
        "desc": "Add or update an easter egg.",
    },
    {
        "type": "ignore",
        "icon": GJ_IGNORE,
        "code": ":see_no_evil:",
        "desc": "Add or update a .gitignore file.",
    },
    {
        "type": "snap",
        "icon": GJ_SNAP,
        "code": ":camera_flash:",
        "desc": "Add or update snapshots.",
    },
    {
        "type": "experiment",
        "icon": GJ_EXPERIMENT,
        "code": ":alembic:",
        "desc": "Perform experiments.",
    },
    {
        "type": "seo",
        "icon": GJ_SEO,
        "code": ":mag:",
        "desc": "Improve SEO.",
    },
    {
        "type": "types",
        "icon": GJ_TYPES,
        "code": ":label:",
        "desc": "Add or update types.",
    },
    {
        "type": "seed",
        "icon": GJ_SEED,
        "code": ":seedling:",
        "desc": "Add or update seed files.",
    },
    {
        "type": "flag",
        "icon": GJ_FLAG,
        "code": ":triangular_flag_on_post:",
        "desc": "Add, update, or remove feature flags.",
    },
    {
        "type": "catch",
        "icon": GJ_CATCH,
        "code": ":goal_net:",
        "desc": "Catch errors.",
    },
    {
        "type": "animation",
        "icon": GJ_ANIMATION,
        "code": ":dizzy:",
        "desc": "Add or update animations and transitions.",
    },
    {
        "type": "deprecation",
        "icon": GJ_DEPRECATION,
        "code": ":wastebasket:",
        "desc": "Deprecate code that needs to be cleaned up.",
    },
    {
        "type": "auth",
        "icon": GJ_AUTH,
        "code": ":passport_control:",
        "desc": "Work on code related to authorization, roles and permissions.",
    },
    {
        "type": "fix-simple",
        "icon": GJ_FIX_SIMPLE,
        "code": ":adhesive_bandage:",
        "desc": "Simple fix for a non-critical issue.",
    },
    {
        "type": "exploration",
        "icon": GJ_EXPLORATION,
        "code": ":monocle_face:",
        "desc": "Data exploration/inspection.",
    },
    {
        "type": "dead",
        "icon": GJ_DEAD,
        "code": ":coffin:",
        "desc": "Remove dead code.",
    },
    {
        "type": "test-fail",
        "icon": GJ_TEST_FAIL,
        "code": ":test_tube:",
        "desc": "Add a failing test.",
    },
    {
        "type": "logic",
        "icon": GJ_LOGIC,
        "code": ":necktie:",
        "desc": "Add or update business logic.",
    },
    {
        "type": "health",
        "icon": GJ_HEALTH,
        "code": ":stethoscope:",
        "desc": "Add or update healthcheck.",
    },
    {
        "type": "infra",
        "icon": GJ_INFRA,
        "code": ":bricks:",
        "desc": "Infrastructure related changes.",
    },
    {
        "type": "devxp",
        "icon": GJ_DEVXP,
        "code": ":technologist:",
        "desc": "Improve developer experience.",
    },
    {
        "type": "money",
        "icon": GJ_MONEY,
        "code": ":money_with_wings:",
        "desc": "Add sponsorships or money related infrastructure.",
    },
    {
        "type": "threading",
        "icon": GJ_THREADING,
        "code": ":thread:",
        "desc": "Add or update code related to multithreading or concurrency.",
    },
    {
        "type": "validation",
        "icon": GJ_VALIDATION,
        "code": ":safety_vest:",
        "desc": "Add or update code related to validation.",
    },
]



import re
from collections import OrderedDict
from pathlib import Path
from typing import Any, Dict, List

from commitizen.cz.base import BaseCommitizen
from commitizen.cz.utils import multiple_line_breaker, required_validator
from commitizen.defaults import MAJOR, MINOR, PATCH

from shared import utils
from shared.gitmojis import *

__all__ = ("CommitizenGitmojiCz",)


def parse_scope(text: str) -> str:
    if not text:
        return ""

    scope = text.strip().split()
    if len(scope) == 1:
        return scope[0]

    return "-".join(scope)


def parse_subject(text: str) -> str:
    if isinstance(text, str):
        text = text.strip(".").strip()

    return required_validator(text, msg="Subject is required.")


class CommitizenGitmojiCz(BaseCommitizen):
    # pattern to match messages for bumping
    # if none of these match, version will not be bumped (unless manually specified)
    bump_pattern = (
        rf"^(BREAKING[\-\ ]CHANGE"
        rf"|{GJ_BOOM}? ?boom"
        rf"|{GJ_FEAT}? ?feat"
        rf"|{GJ_FIX}? ?fix"
        rf"|{GJ_HOTFIX}? ?hotfix"
        rf"|{GJ_REFACTOR}? +refactor"
        rf"|{GJ_PERF}? ?perf)"
        r"(\(.+\))?"  # scope
        r"(!)?"  # breaking
    )
    # map types to SemVer
    bump_map = OrderedDict(
        (
            (r"^.+!$", MAJOR),
            (r"^BREAKING[\-\ ]CHANGE", MAJOR),
            (rf"^{GJ_BOOM}? ?boom", MAJOR),
            (rf"^{GJ_FEAT}? ?feat", MINOR),
            (rf"^{GJ_FIX}? ?fix", PATCH),
            (rf"^{GJ_HOTFIX}? ?hotfix", PATCH),
            (rf"^{GJ_REFACTOR}? ?refactor", PATCH),
            (rf"^{GJ_PERF}? ?perf", PATCH),
        )
    )
    # parse information for generating the change log
    commit_parser = (
        rf"^(?P<change_type>{utils.get_type_group_pattern()}|BREAKING CHANGE)"
        r"(?:\((?P<scope>[^()\r\n]*)\)|\()?(?P<breaking>!)?:\s(?P<message>.*)?"
    )
    # exclude from changelog
    changelog_pattern = (
        rf"^(?!{GJ_INIT}? ?init)(?!{GJ_MERGE}? ?merge)(?!{GJ_BUMP}? ?bump).*"
    )
    # map types to changelog sections
    change_type_map = {
        # boom
        f"{GJ_BOOM} boom": f"{GJ_BOOM} Boom",
        # features
        f"{GJ_FEAT} feat": f"{GJ_FEAT} Features",
        # fixes
        f"{GJ_FIX} fix": f"{GJ_FIX}{GJ_HOTFIX} Fixes",
        f"{GJ_HOTFIX} hotfix": f"{GJ_FIX}{GJ_HOTFIX} Fixes",
        # security
        f"{GJ_SECURITY} security": f"{GJ_SECURITY} Security",
        # license
        f"{GJ_LICENSE} license": f"{GJ_LICENSE} License",
        # refactorings
        f"{GJ_REFACTOR} refactor": f"{GJ_REFACTOR} Refactorings",
        # style & architecture
        f"{GJ_STYLE} style": f"{GJ_STYLE}{GJ_ARCH} Style & Architecture",
        f"{GJ_ARCH} arch": f"{GJ_STYLE}{GJ_ARCH} Style & Architecture",
        # performance
        f"{GJ_PERF} perf": f"{GJ_PERF} Performance",
        # docs
        f"{GJ_DOCS} docs": f"{GJ_DOCS}{GJ_SOURCE_DOCS} Documentation",
        f"{GJ_SOURCE_DOCS} source-docs": f"{GJ_DOCS}{GJ_SOURCE_DOCS} Documentation",
        # tests
        f"{GJ_TEST} test": f"{GJ_TEST}{GJ_MOCK}{GJ_TEST_FAIL} Tests",
        f"{GJ_MOCK} mock": f"{GJ_TEST}{GJ_MOCK}{GJ_TEST_FAIL} Tests",
        f"{GJ_TEST_FAIL} test-fail": f"{GJ_TEST}{GJ_MOCK}{GJ_TEST_FAIL} Tests",
        # auth
        f"{GJ_AUTH} auth": f"{GJ_AUTH} Authentication",
        # ci & build
        f"{GJ_BUILD} build": f"{GJ_CI}{GJ_BUILD} CI & Build",
        f"{GJ_CI} ci": f"{GJ_CI}{GJ_BUILD} CI & Build",
        # ui & uix
        f"{GJ_UI} ui": f"{GJ_UI}{GJ_UX} UI & UIX",
        f"{GJ_UX} ux": f"{GJ_UI}{GJ_UX} UI & UIX",
        # configuration & scripts & packages
        f"{GJ_CONFIG} config": f"{GJ_CONFIG}{GJ_SCRIPT}{GJ_PACKAGE} Configuration, Scripts, Packages",
        f"{GJ_SCRIPT} script": f"{GJ_CONFIG}{GJ_SCRIPT}{GJ_PACKAGE} Configuration, Scripts, Packages",
        f"{GJ_PACKAGE} package": f"{GJ_CONFIG}{GJ_SCRIPT}{GJ_PACKAGE} Configuration, Scripts, Packages",
        # cleanup
        f"{GJ_DUMP} dump": f"{GJ_DUMP}{GJ_DEAD} Clean up",
        f"{GJ_DEAD} dead": f"{GJ_DUMP}{GJ_DEAD} Clean up",
        # dependencies
        f"{GJ_DEP_ADD} dep-add": f"{GJ_PIN}{GJ_DEP_ADD}{GJ_DEP_DROP}{GJ_DEP_RM}{GJ_DEP_BUMP} Dependencies",
        f"{GJ_DEP_RM} dep-rm": f"{GJ_PIN}{GJ_DEP_ADD}{GJ_DEP_DROP}{GJ_DEP_RM}{GJ_DEP_BUMP} Dependencies",
        f"{GJ_DEP_BUMP} dep-bump": f"{GJ_PIN}{GJ_DEP_ADD}{GJ_DEP_DROP}{GJ_DEP_RM}{GJ_DEP_BUMP} Dependencies",
        f"{GJ_DEP_DROP} dep-drop": f"{GJ_PIN}{GJ_DEP_ADD}{GJ_DEP_DROP}{GJ_DEP_RM}{GJ_DEP_BUMP} Dependencies",
        f"{GJ_PIN} pin": f"{GJ_PIN}{GJ_DEP_ADD}{GJ_DEP_DROP}{GJ_DEP_RM}{GJ_DEP_BUMP} Dependencies",
        # types
        f"{GJ_TYPES} types": f"{GJ_TYPES} Types",
        # language & accessibility
        f"{GJ_LANG} lang": f"{GJ_LANG}{GJ_ACCESSIBILITY} Language & Accessibility",
        f"{GJ_ACCESSIBILITY} accessibility": f"{GJ_LANG}{GJ_ACCESSIBILITY} Language & Accessibility",
        # design
        f"{GJ_DESIGN} design": f"{GJ_DESIGN}{GJ_ANIMATION} Design",
        f"{GJ_ANIMATION} animation": f"{GJ_DESIGN}{GJ_ANIMATION} Design",
        # logs
        f"{GJ_LOGS_ADD} logs-add": f"{GJ_LOGS_ADD}{GJ_LOGS_RM} Logs",
        f"{GJ_LOGS_RM} logs-rm": f"{GJ_LOGS_ADD}{GJ_LOGS_RM} Logs",
        # people
        f"{GJ_PEOPLE} people": f"{GJ_PEOPLE} People",
        # database
        f"{GJ_DB} db": f"{GJ_DB} Database",
        # ignore
        f"{GJ_IGNORE} ignore": f"{GJ_IGNORE} Ignore",
        # snapshots
        f"{GJ_SNAP} snap": f"{GJ_SNAP} Snapshots",
        # experiments
        f"{GJ_EXPERIMENT} experiment": f"{GJ_EXPERIMENT} Experiments",
        # SEO
        f"{GJ_SEO} seo": f"{GJ_SEO} SEO",
        # infra
        f"{GJ_INFRA} infra": f"{GJ_INFRA} Infrastructure",
        # devxp
        f"{GJ_DEVXP} devxp": f"{GJ_DEVXP} Developer Experience",
        # money
        f"{GJ_MONEY} money": f"{GJ_MONEY} Sponsoring",
        # threading
        f"{GJ_THREADING} threading": f"{GJ_THREADING} Threading",
        # validation
        f"{GJ_VALIDATION} validation": f"{GJ_VALIDATION} Validation",
        # revert
        f"{GJ_REVERT} revert": f"{GJ_REVERT} Reversions",
        # deploy
        f"{GJ_DEPLOY} deploy": f"{GJ_DEPLOY} Deployments",
        # linting
        f"{GJ_FIX_LINT} fix-lint": f"{GJ_FIX_LINT} Linting",
        # resource & assets
        f"{GJ_RESOURCE} resource": f"{GJ_RESOURCE}{GJ_ASSET} Resources & Assets",
        f"{GJ_ASSET} asset": f"{GJ_RESOURCE}{GJ_ASSET} Resources & Assets",
        # others
        f"{GJ_SECRET} secret": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_WIP} wip": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_ANALYTICS} analytics": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_TYPO} typo": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_POOP} poop": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_EXTERNAL} external": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_BEER} beer": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_TEXT} text": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_EGG} egg": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_SEED} egg": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_FLAG} egg": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_CATCH} egg": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        f"{GJ_HEALTH} health": f"{GJ_SECRET}{GJ_WIP}{GJ_ANALYTICS}{GJ_TYPO}{GJ_POOP}{GJ_EXTERNAL}{GJ_BEER}{GJ_TEXT}{GJ_EGG}{GJ_SEED}{GJ_FLAG}{GJ_CATCH}{GJ_HEALTH} Others",
        # None: init, bump, merge
    }

    def questions(self) -> List[Dict[str, Any]]:
        """Return the questions to ask the user."""
        return [
            {
                "type": "list",
                "name": "prefix",
                "message": "Select the type of change you are committing",
                "choices": [
                    {
                        "value": moji.value,
                        "name": moji.name,
                    }
                    for moji in utils.get_gitmojis()
                ],
            },
            {
                "type": "input",
                "name": "scope",
                "message": (
                    "What is the scope of this change? (class or file name): "
                    "(press [enter] to skip)\n"
                ),
                "filter": parse_scope,
            },
            {
                "type": "input",
                "name": "subject",
                "filter": parse_subject,
                "message": (
                    "Write a short and imperative summary of the code changes: "
                    "(lower case and no period)\n"
                ),
            },
            {
                "type": "input",
                "name": "time",
                "message": "Time spent (i.e. 3h 15m) (press [enter] to skip):\n",
                "filter": lambda x: "⏰ " + x.strip() if x else "",
            },
            {
                "type": "input",
                "name": "body",
                "message": (
                    "Provide additional contextual information about the code changes: "
                    "(press [enter] to skip)\n"
                ),
                "filter": multiple_line_breaker,
            },
            {
                "type": "confirm",
                "message": "Is this a BREAKING CHANGE? Correlates with MAJOR in SemVer",
                "name": "is_breaking_change",
                "default": False,
            },
            {
                "type": "input",
                "name": "footer",
                "message": (
                    "Footer. Information about Breaking Changes and reference "
                    "issues that this commit closes: (press [enter] to skip)\n"
                ),
            },
        ]

    def message(self, answers: Dict[str, Any]) -> str:
        """Generate a commit message from the answers."""
        prefix = answers["prefix"]
        scope = answers["scope"]
        subject = answers["subject"]
        body = answers["body"]
        footer = answers["footer"]
        is_breaking_change = answers["is_breaking_change"]
        time = answers["time"]

        if scope:
            scope = f"({scope})"
        if body:
            body = f"\n\n{body}"
        if is_breaking_change and footer and not footer.startswith("BREAKING CHANGE"):
            footer = f"BREAKING CHANGE: {footer}"
        if footer:
            footer = f"\n\n{footer}"
        if time:
            time = f" >>> {time}"

        prefix_scope = f"{prefix}{scope}"
        if is_breaking_change:
            prefix_scope = f"{prefix_scope}!"

        message = f"{prefix_scope}: {subject}{time}{body}{footer}"

        return message

    def example(self) -> str:
        """Return an example commit message."""
        return (
            f"{GJ_FIX} fix: correct minor typos in code\n"
            "\n"
            "see the issue for details on the typos fixed\n"
            "\n"
            "closes issue #12"
        )

    def schema(self) -> str:
        """Return the commit message schema."""
        return (
            "<gitmoji><type>(<scope>): <subject>\n"
            "<BLANK LINE>\n"
            "<body>\n"
            "<BLANK LINE>\n"
            "(BREAKING CHANGE: )<footer>"
        )

    # pattern to validate commits
    def schema_pattern(self) -> str:
        """Return the schema validation pattern."""
        return utils.get_pattern()

    def info(self) -> str:
        """Return information about the commit message style."""
        dir_path = Path(__file__).parent
        filepath = dir_path.joinpath("conventional_gitmojis_info.txt")
        with open(filepath, "r") as f:
            content = f.read()
        return content

    def process_commit(self, commit: str) -> str:
        """Process a commit."""
        pat = self.schema_pattern()
        m = re.match(pat, commit)
        if m is None:
            return ""
        return m.group(3).strip()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src', 'cz_gitmoji': 'src/cz_gitmoji', 'gitmojify': 'src/gitmojify'}

packages = \
['cz_gitmoji', 'gitmojify', 'shared']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=22.2.0,<23.0.0', 'commitizen>=2.38.0,<3.0.0']

entry_points = \
{'console_scripts': ['gitmojify = gitmojify.mojify:run']}

setup_kwargs = {
    'name': 'cz-conventional-gitmoji',
    'version': '0.1.1',
    'description': 'A commitizen plugin that combines gitmoji and conventional.',
    'long_description': '# cz-conventional-gitmoji\n\nA [commitizen](https://github.com/commitizen-tools/commitizen) plugin that combines [gitmoji](https://gitmoji.dev/) and [conventional commits](https://www.conventionalcommits.org/en/v1.0.0/).\n\n## Installation\n\n```bash\npoetry add cz-conventional-gitmoji\n```\n\nor with pip:\n\n```bash\npip install cz-conventional-gitmoji\n```\n\n## Usage\n\nThis package can be used as a normal `commitizen` plugin, either by specifying the name on the command line\n\n```bash\ncz --name cz_gitmoji commit\n```\n\nor by setting it in your **pyproject.toml**\n\n```toml\n[tool.commitizen]\nname = "cz_gitmoji"\n```\n\nThis will make `commitizen` use the commit message parsing rules defined by this plugin, which are 100% compatible with [conventional commits](https://www.conventionalcommits.org/en/v1.0.0/). As such, the gitmojis are completely optional and all commands will continue to validate commit messages in conventional format just fine. This is useful if you\'re transitioning an existing repo to `contentional-gitmoji` or you work in a team in which some colleagues don\'t like gitmojis.\n\n### gitmojify\n\nApart from the conventional-gitmoji rules, this package provides the `gitmojify` command which is also available as a pre-commit hook. The command reads a commit message either from cli or a commit message file and prepends the correct gitmoji based on the type. If the message already has a gitmoji, it is returned as is.\n\n```bash\n$ gitmojify -m "init: initial version"\n🎉 init: initial version\n```\n\nTo use it as a pre-commit hook, install this packages as well as `commitizen` and put the following into your **.pre-commit-config.yaml**\n\n```yaml\nrepos:\n  - repo: https://github.com/ljnsn/cz-conventional-gitmoji\n    rev: main\n    hooks:\n      - id: conventional-gitmoji\n```\n\nCommit with a message in conventional format that contains a valid type mapped by conventional gitmoji and the gitmoji will automagically be added.\n\n## Features\n\n- [x] Enable conventional gitmoji commit messages via `cz commit`.\n- [ ] Add `--simple-emojis` option to use only the emojis relating to `cz_conventional_commits` types.\n- [ ] Add `--simple-types` option to use only the types used by `cz_conventional_commits`.\n- [ ] Add `--conventional` option to put the emoji in the commit message, making it compatible with `cz_conventional_commits`.\n- [x] Add hook to automatically prepend the appropriate gitmoji for the commit\'s type.\n\n## Inspiration\n\n- [`commitizen-emoji`](https://github.com/marcelomaia/commitizen-emoji)\n',
    'author': 'ljnsn',
    'author_email': '82611987+ljnsn@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ljnsn/cz-conventional-gitmoji',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import argparse
import re
import sys
from pathlib import Path
from typing import Dict

from shared.model import Gitmoji
from shared.utils import get_gitmojis, get_pattern


def get_args() -> argparse.Namespace:
    """Parse command line arguments."""
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("-f", "--commit-msg-file")
    group.add_argument("-m", "--message")
    return parser.parse_args()


def grouped_gitmojis() -> Dict[str, Gitmoji]:
    """Return the gitmojis grouped by type."""
    return {moji.type: moji for moji in get_gitmojis()}


def gitmojify(message: str) -> str:
    """
    Gitmojify the commit message.

    If a gitmoji is already present in the message, the message is returned
    as is. Otherwise, the gitmoji is looked up by type and is prepended to
    the message.

    Args:
        message: The complete commit message.

    Returns:
        The gitmojified message.
    """
    pat = re.compile(get_pattern())
    match = pat.match(message)
    if match is None:
        raise ValueError("invalid commit message")
    gtype = match.group(1)
    if " " in gtype:  # maybe do a better check?
        return message
    icon = grouped_gitmojis()[gtype].icon
    return f"{icon} {message}"


def run() -> None:
    """The pre-commit hook that modifies the commit message."""
    args = get_args()
    if args.commit_msg_file:
        filepath = Path(args.commit_msg_file)
        msg = filepath.read_text()
        with filepath.open("w") as f:
            f.write(gitmojify(msg))
    else:
        msg = args.message
        sys.stdout.write(gitmojify(msg))
